#ifndef __CONTROLLER_H
#define __CONTROLLER_H

#include <ishtar/settings.h>
#include <vector>

/*!	\file Controller.h
	\brief Header of the Controller. Generic class that is capable to map some input to some output.
*/

/*!	\defgroup controllers Controllers and neural networks
	Anything that read inputs and produce outputs
*/

namespace Teem
{
	//! \ingroup controllers
	//! Interface for a controller, which is anything that read inputs and produce outputs
	class Controller
	{
	public:
		//! Constructor, takes the Ishtar root as a parameter
		Controller(const std::string &root = "teem.controller") { }
		//! Virtual destructor, do nothing
		virtual ~Controller() { }
		
		//! Set the input @p index to value @p val
		virtual void setInput(unsigned index, double val) = 0;
		// here: other setInput overloads to ease usage
		
		//! Return the value of output @p index
		virtual double getOutput(unsigned index) = 0;
		// here: other getOutput overloads to ease usage
		
		//! Do a step, process inputs and compute outputs
		virtual void step() = 0;
	};
}

#endif
