#include "MatrixController.h"
#include "../ga/Genome.h"
#include "../ga/ValueGenome.h"

/*!	\file MatrixController.cpp
	\brief Implementation of the matrix controller
*/

namespace Teem
{
	MatrixController::MatrixController(unsigned inputCount, unsigned outputCount)
		: matrix(inputCount, outputCount), bias(outputCount), input(inputCount), output(outputCount)
	{
	}
	
	void MatrixController::step()
	{
		output = matrix * input + bias;
	}
	
	
	Genome* MatrixGenotypeDecoder::createGenome()
	{
		unsigned geneCount = inputCount * outputCount;
		if (evolveBias)
			geneCount += outputCount;
		Genome* genome = Genome::create(Genome::BIT_GENOME, geneCount * 8);
		genome->randomize();
		return genome;
	}
	
	Controller* MatrixGenotypeDecoder::decode(Genome* genome)
	{
		ValueGenome *vg = dynamic_cast<ValueGenome*>(genome);
		if (!vg) return NULL;
		
		// create controller
		MatrixController* mc = new MatrixController(inputCount, outputCount);
		
		// fill the matrix coefficients
		for (size_t x = 0; x < inputCount; x++)
		{
			for(size_t y = 0; y < outputCount; y++)
				mc->matrix(x, y) = vg->getDouble(-coefRange, coefRange);
		}
		
		// fill the bias
		if (evolveBias)
			for (size_t x = 0; x < outputCount; x++)
				mc->bias[x] = vg->getDouble(-coefRange, coefRange);
		
		vg->resetReading();
		
 		return mc;
	}
}
