#ifndef __SIGMOIDNEURON_H
#define __SIGMOIDNEURON_H

#include "Neuron.h"

/*!	\file SigmoidNeuron.h
	\brief Interface of sigmoid neurons
*/

namespace Teem
{
	//! Sigmoidal (atan) neuron
	//! \ingroup neurons
	class SigmoidNeuron : public Neuron
	{
	public:
		SigmoidNeuron();
		virtual ~SigmoidNeuron() {}
		
		virtual void setParams(const std::string &param, double value);
		virtual void update(double dt);
		virtual void commit();

	protected:
		Ishtar::Variable<double> alpha; //!< alpha value, used in:\n \f$lastCurrent = tanh(alpha \cdot \sum_{i=0}^{incoming.size()-1} ( incoming[i].getCurrent() \cdot incoming[i].weight ))\f$
		
		double activation; //!< activation state of this neuron
	};
	
	//! Simple input neuron that filters its value through the sigmoid
	//! activation function.
	//! \ingroup neurons
	class SigmoidInputNeuron :  public InputNeuron
	{
	public:
		//! Constructor
		SigmoidInputNeuron() { }
		virtual ~SigmoidInputNeuron() { }
		
		virtual void setInputValue(double value) { lastCurrent = value; }
	};
}

#endif
