#ifndef __BATTERY_RECHARGE
#define __BATTERY_RECHARGE

#ifdef HAVE_ENKI

#include "SingleRobotExperiment.h"
#include "../world/World.h"
#include "../world/Entity.h"
#include "../utils/JustDoIt.h"
#include "../ga/GenerationalEvolution.h"

#include <an/Formula.h>
#include <an/Types.h>

#include <valarray>
#include <string>

namespace Teem
{
	class OctaveLogger;
	class Controller;
	
	//! Demonstration and education experiment recreating the battery recharging setup.
	class BatteryRecharge : public SingleRobotExperiment, public JustDoIt
	{
		friend class BatteryRechargeEnkiViewer;
		
	protected:
	
		SquareArenaWorld *w;		//!< world used by the experiment
		Khepera* robot;				//!< pointer to the robot
		
		Ishtar::Variable<bool, true, 8> inputIR;			//!< use IR as input? (starting left then clockwise)
		Ishtar::Variable<bool, true, 2> inputEncoders;		//!< use encoder as input? (left, right)
		Ishtar::Variable<bool, true> inputBatteryLevel;		//!< use battery level as input?
		Ishtar::Variable<bool, true, 2> inputAmbiant;		//!< use ambiant light sensors as input?
		Ishtar::Variable<bool, true> inputFloor;			//!< use the floor sensor as input?
		
		Ishtar::Variable<double> batteryCapacity;			//!< max battery capacity
		Ishtar::Variable<double> initialBatteryCapacity;	//!< initial battery capacity
		Ishtar::Variable<double> maxSpeed;					//!< max speed of the robot
		
		Ishtar::Variable<double> rechargeZoneRadius;		//!< radius of the recharging zone (centered in 0;0)
		Ishtar::Variable<std::string, true> fitnessFunction;	//!< fitness function to use
		
		/* other desirable settings
		
		- fitness function
		- network topology (recurency on output layer)
		*/
	
		class World;
		
	public:
		//! Constructor
		BatteryRecharge();
		//! Destructor.
		virtual ~BatteryRecharge() { }
		
		virtual void setupDecoders(GenotypeDecoders &decoders);
		virtual void doIt();
	
	protected:
	
		Controller* controller;
		OctaveLogger* logger;
		An::Formula fitnessFormula;
		bool joystick;
		double leftJoystick, rightJoystick;
		
		// log and formula variables updated at the beginning of the time-step
		std::valarray<double> ir;
		double leftEncoder, rightEncoder, leftMotor, rightMotor;
		double orientation;
		An::Point pos;
		double batteryLevel;
		double frontAmbiant, backAmbiant;
		double floorSensor;
		
		// SingleRobotExperiment overrides	
		virtual void createControllers(GenotypeDecoders &decoders, GenerationalPopulation &individuals, unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode);
		virtual void initEpoch(unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode);
		virtual void termEpoch(unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode);
		virtual double doSensoryMotorCycle(unsigned cycle, bool &kill);
		
		//! Assign inputs to the controller. The function uses the class variables that should contain up-to-date values.
		void assignInputsToController();
		
		//! Compute the current fitness. The function uses the class variables that should contain up-to-date values. 
		double computeFitness();
		
		//! Simulate the ambiant light sensors 
		double simulateAmbiant(double xOffset, double yOffset, double angleOffset);
	};
}

#endif //--HAVE_ENKI

#endif
