#include "SingleRobotExperiment.h"

#include "../main/VerboseLevel.h"


namespace Teem
{
	SingleRobotExperiment::SingleRobotExperiment() :
		epochCount("teem.experiment.epochCount", 2),
		epochDuration("teem.experiment.epochDuration", 120),
		simulationStep("teem.experiment.simulationStep", 0.05),
		cycleCount((double) epochDuration / simulationStep)
	{
	}
	
	void SingleRobotExperiment::compute(GenotypeDecoders &genotypeDecoders, GenerationalPopulation &individuals, unsigned gen, unsigned ind, EvaluationMode mode)
	{
		double fitness = 0.0;
		
		// print info to command line
		COUT(3, "@begin_ind " << gen << " " << ind << std::endl);
		if(verboseLevel == 2)
		{
			// This is done only using the default verbosity of 2
			if(ind == 0)
			{
				std::cout << std::endl << "Generation " << gen << " : ";
				std::cout.flush();
			}
			if(ind % 10 == 0)
			{
				std::cout << ".";
				std::cout.flush();
			}
		}
		
		for(unsigned epoch = 0; epoch < epochCount; epoch++)
		{
			createControllers(genotypeDecoders, individuals, gen, ind, epoch, mode);
			initEpoch(gen, ind, epoch, mode);
			fitness += doEpoch(gen, ind, epoch, mode);
			termEpoch(gen, ind, epoch, mode);
		}
		
		fitness /= epochCount;
		individuals[0].fitness = fitness;
		
		COUT(3, "@fitness " << fitness << std::endl << "@end_ind" << std::endl);
	}
	
	double SingleRobotExperiment::doEpoch(unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode)
	{
		double fitness = 0.0;
		for(unsigned cycle = 0; cycle < cycleCount; cycle++)
		{
			bool kill = false;
			fitness += doSensoryMotorCycle(cycle, kill);
			if(kill)
			{
				COUT(3, "@killed" << std::endl);
				break;
			}
			
			// step ishtar
			if ((cycle & (~0xFF)) == 0)
				Ishtar::settings->step();
		}
		fitness /= cycleCount;
		
		COUT(3, "@epoch_fitness " << epoch << " " << fitness << std::endl);
		return fitness;
	}
}
