#ifndef __SVGPLOTTER_H
#define __SVGPLOTTER_H

#include <iostream>
#include <fstream>
#include <vector>

/*!	\file SVGPlotter.h
	\brief Interface of a simple SVG renderer
*/
namespace Teem
{
	//! A plotter that can draw trajectories in SVG
	//! \ingroup utils
	class SVGPlotter
	{
	protected:
		//! File on which to export the SVG
		std::ofstream outputFile;
		//! Last point for each trajectory
		std::vector<std::pair<double, double> > trajectories;
	public:
		//! Create the plotter. Plot to file
		SVGPlotter(const std::string &filename, double width, double height);
		//! Destructor, close file
		virtual ~SVGPlotter();
		//! Create a trajectory starting from point (x,y). Returns the trajectory id
		size_t createTrajectory(double x, double y);
		//! Draw a new line for a given trajectory
		void drawLine(size_t trajectory, double x, double y, double r, double g, double b);
	};
}

#endif
