/*
  This code has been taken from Globulation 2 (http://glob2.ysagoon.com)
  
  Copyright (C) 2001-2005 Stephane Magnenat & Luc-Olivier de Charrière
  for any question or comment contact us at nct@ysagoon.com or nuage@ysagoon.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __VFS_H
#define __VFS_H

#include <string>
#include <vector>

/*!	\file VFS.h
	\brief Interface of a simple virtual filesystem
*/

namespace Teem
{
	//! Very simple and lightweight virtual filesystem
	//! \ingroup utils
	class VFS
	{
	private:
		//! List of directory where to search for requested file
		std::vector<std::string> dirList;
		
	public:	
		//! VFS constructor, takes applicationName in parameter, to search in ~/.applicationName
		VFS(/*const std::string &applicationName*/);
		//! VFS destructor
		virtual ~VFS();
	
		//! Add a directory to the search list
		void addDir(const std::string &dir);
		//! Add a new subdir (create it if needed) which will be used to open file in write mode in it
		void addWriteSubdir(const std::string &subdir);
		//! Return the first valid full path for file name. Returns "" if no such path exist
		std::string getFullPath(const std::string &name, const std::string &mode = "r");
	};
	
	//! Global instance
	//! \ingroup utils
	extern VFS vfs;
}

#endif
